-- Program: ColorTest
-- Purpose: An ugly program that demonstrates use of 
--          colors and drawing capabilities of Gtk+HS   
-- Written by: Shuying Wang, 2002 (copylefted :-p )
-- Compiled with: gtk+hs-0.11.5,ghc 5.02
-- to compile: ghc -o ColorTest ColorTest.hs
-- complaints and suggestions email: swan936@cse.unsw.edu.au

import IOExts (IORef, newIORef, writeIORef, readIORef)
import Maybe (fromJust)
import System (exitWith, ExitCode(..))

import Gdk 
import Gtk hiding (main, init)
import qualified 
       Gtk (main, init)
--import GtkTypeUtils

-- constants
-- ---------
startp :: Int
startp = 0 -- starting angle of a circle

endp :: Int
endp = 360* 64 -- ending angle of a circle 

colString :: [String]
colString = ["LightGreen","DarkRed","DarkMagenta","DarkCyan","DarkBlue",
	  "DarkGrey","thistle4","MediumPurple4","purple1", "DarkOrchid1",
           "plum1","VioletRed1","HotPink1","DeepPink1","OrangeRed1",
	  "tomato1","coral1","DarkOrange1","salmon1","brown1","firebrick1",
	  "chocolate1","tan1","wheat1","sienna1","IndianRed1","RosyBrown1",
	  "DarkGoldenrod1","goldenrod1","gold1","LightYellow1",
	  "LightGoldenrod1","khaki1","DarkOliveGreen1","Chartreuse1",
	  "green1","SpringGreen1","PaleGreen1","SeaGreen1","DarkSeaGreen1",
	  "aquamarine1","DarkSlateGray1", "cyan1","turquoise1","CadetBlue1",
	  "PaleTurquoise1","LightCyan1","LightBlue1","LightSteelBlue1",
	  "SlateGray1","LightSkyBlue1","SkyBlue1","DeepSKyBlue1","SteelBlue1",
	  "DodgerBlue1","blue1","RoyalBlue1","SlateBlue1","azure1",
	  "MistyRose1","LavenderBlush1","honeydew1","ivory","cornsilk",
	  "LemonChiffon","NavajoWhite","PeachPuff","bisque","AntiqueWhite",
	  "seashell","snow","BlueViolet","maroon","SaddleBrown","beige",
	  "burlywood","sandy brown","wheat","peru","ForestGreen","LimeGreen",
	  "LawnGreen","green","PowderBlue","CornflowerBlue","NavyBlue",
	  "navy","MidnightBlue","gray","grey","DimGrey","AliceBlue",
	  "MintCream","moccasin","BlanchedAlmond","PapayaWhip","linen",
	  "OldLace","FloralWhite","gainsboro","GhostWhite","WhiteSmoke",
          "ForestGreen"]

-- auxilliary functions
-- ---------------------
drawCircle :: Drawable -> GC 
             -> Bool -> (Int,Int) -> Int -> IO ()
drawCircle draw gc fill (x1,y1) diameter = 
         drawArc draw gc fill x1 y1 diameter diameter startp endp

drawCircles :: Drawable -> GC
                -> Bool -> [(Int,Int)] -> IO ()
drawCircles draw gc fill [] = return ()
drawCircles draw gc fill (coord:rest) = do
        drawCircle draw gc fill coord 20 
        drawCircles draw gc fill rest 

-- drawing callbacks
-- -----------------
drawobjs :: Drawable -> GC  
            -> IORef [String] -> [(Int,Int)] -> IO Bool
drawobjs draw gc cols pos = do
  colList <- readIORef cols
  case colList of
    []         -> return False
    (col:rest) -> do
      writeIORef cols rest
      case colorParse col of
        Nothing      -> do
			  putStrLn ("Don't know color `" ++ col ++ "'")
			  return False
        Just fgcolor -> do
			  cm <- colormapGetSystem
			  fcolor <- colormapAllocColor cm fgcolor False True
			  gcSetForeground gc (fromJust fcolor)
			  drawCircles draw gc True pos
			  return True

paints :: DrawingArea -> [(Int,Int)] -> IO Bool
paints da pos = do
     state <- newIORef colString
     draw <- widgetGetWindow da
     gc   <- gcNew draw
     paint draw gc state pos
     return True
      where
        paint :: Drawable -> GC -> IORef [String] 
                  -> [(Int,Int)] -> IO (Connection TimeoutHandler)
        paint draw gc cols pos = 
          let hdl = TimeoutHandler (drawobjs draw gc cols pos)
          in
          timeoutAdd 1000 hdl


--  -----------------
       
Main.main :: IO ()
main = do
         (_,args) <- Gtk.init Nothing

         window <- windowNew WindowToplevel
         windowSetTitle window "Color test"
         windowSetDefaultSize window 380 380
         let winDelHdl = WidgetDeleteEventHandler $ \_ _ -> do
							    mainQuit
							    return False
	 signalConnect window winDelHdl
         containerBorderWidth window 5

         drawArea <- drawingAreaNew
         let ys = [0,80..400]
             xs = [0,80..400] 
             pos = [(x,y)|x <- xs, y <- ys]
             drawHdl = WidgetExposeEventHandler $
                              \da _ -> paints da pos 
         signalConnect drawArea drawHdl
         containerAdd window drawArea

         -- widgetShow drawArea
         widgetShowAll window

         Gtk.main
	 exitWith ExitSuccess

