-- program to test some of the entry widget's functionality
--

import System (getArgs, ExitCode(ExitSuccess), exitWith)
import Gtk    (WindowType(..),
               ObjectSignalHandler(..), 
	       Box, boxPackStartDefaults,
	       HBox, hBoxNew,
	       VBox, vBoxNew,
	       Button, buttonNewWithLabel,
	       ButtonSignalHandler(..), signalConnect,
	       CheckButton, checkButtonNewWithLabel,
	       Container, containerBorderWidth, containerAdd,
	       EditableSignalHandler(..), 
	       Entry, entryGetText, entrySetVisibility, 
	       entrySetEditable, entryNew, entrySetMaxLength, 
	       entrySetText, entryAppendText, entrySelectRegion,
	       ToggleButton, toggleButtonNewWithLabel,
	       toggleButtonQueryState, toggleButtonSetState,
	       ToggleButtonSignalHandler(..), 
	       Widget, widgetShow, widgetDestroy, widgetSetUSize,
	       WidgetSignalHandler(..), 
	       Window, windowNew, windowSetTitle,
	       mainQuit)
import qualified
       Gtk    (init, main)


enter     :: Entry -> IO ()
enter ety  = do
	       text <- entryGetText ety
	       putStr ("Entry contents: " ++ text ++ "\n")

toggleEditable         :: Entry -> CheckButton -> IO ()
toggleEditable ety cbt  = do
			    flag <- toggleButtonQueryState cbt
			    entrySetEditable ety flag

toggleVisibility         :: Entry -> CheckButton -> IO ()
toggleVisibility ety cbt  = do
			    flag <- toggleButtonQueryState cbt
			    entrySetVisibility ety flag

main :: IO ()
main  = do
	  Gtk.init Nothing

	  window <- windowNew WindowToplevel
	  windowSetTitle window "GTK+ Entry"
	  widgetSetUSize window 200 100
	  let winDelHdl = WidgetDeleteEventHandler $ \_ _ -> do
							       mainQuit
							       return False
	  signalConnect window winDelHdl
				 
	  containerBorderWidth window 10

	  -- box for the whole thing
	  --
	  bigBox <- vBoxNew False 10
	  containerAdd window bigBox
	  widgetShow bigBox

	  entry <- entryNew
	  entrySetMaxLength entry 50
	  let actHdl = EditableActivateHandler enter
	  signalConnect entry actHdl
	  entrySetText entry "hello"
	  entryAppendText entry " world"
	  fullText <- entryGetText entry
	  entrySelectRegion entry 0 (length fullText)
	  boxPackStartDefaults bigBox entry
	  widgetShow entry

	  -- box for assembling buttons
	  --
	  butBox <- hBoxNew False 10
	  containerAdd bigBox butBox
	  widgetShow butBox

	  -- button to toggle editable status
	  --
	  checkEdit <- checkButtonNewWithLabel " Editable "
	  boxPackStartDefaults butBox checkEdit
	  toggleButtonSetState checkEdit True -- should be after the handler
						 -- but _call_GC_ doesn't work
	  let editHdl = ToggleButtonToggledHandler (toggleEditable entry)
	  signalConnect checkEdit editHdl
	  widgetShow checkEdit

	  -- button to toggle visibility status
	  --
	  checkVisi <- checkButtonNewWithLabel " Visible "
	  boxPackStartDefaults butBox checkVisi
	  toggleButtonSetState checkVisi True -- should be after the handler
						 -- but _call_GC_ doesn't work
	  let visiHdl = ToggleButtonToggledHandler (toggleVisibility entry)
	  widgetShow checkVisi

	  buttonQuit <- buttonNewWithLabel " Quit "
	  let quitHandler wd = widgetDestroy wd >> mainQuit
	  signalConnect
	    buttonQuit (ButtonClickedHandler (const (quitHandler window)))
	  boxPackStartDefaults bigBox buttonQuit
	  widgetShow buttonQuit

	  widgetShow window

	  Gtk.main
	  exitWith ExitSuccess
