-- test for range widgets (preliminary version - without menus)
--

import System (getArgs, ExitCode(ExitSuccess, ExitFailure), exitWith)
import Monad  (when)
import Gtk    hiding (init, main)
import qualified
       Gtk    (init, main)


scaleSetDefaultValues       :: ScaleClass scl => scl -> IO ()
scaleSetDefaultValues scale  =
  do
    rangeSetUpdatePolicy scale UpdateContinuous
    scaleSetDigits scale 1
    scaleSetValuePos scale PosTop
    scaleSetDrawValue scale True

createRangeControl :: IO ()
createRangeControl  =
  do
    window <- windowNew WindowToplevel
    windowSetTitle window "Range widget test..."
    let winDelHdl = WidgetDeleteEventHandler $ \_ _ -> do
							 mainQuit
							 return False
    signalConnect window winDelHdl

    -- box for the whole thing
    --
    bigBox <- vBoxNew False 0
    containerAdd window bigBox
    widgetShow bigBox

    -- box for vscale
    --
    topHBox <- hBoxNew False 10
    containerBorderWidth topHBox 10
    boxPackStartDefaults bigBox topHBox
    widgetShow topHBox

    -- value, lower, upper, step_increment, page_increment, page_size 
    -- note that the page_size value only makes a difference for
    -- scrollbar widgets, and the highest value you'll get is actually
    -- (upper - page_size)
    --
    adj1 <- adjustmentNew 0.0 0.0 101.0 0.1 1.0 1.0
    upper <- adjustmentGetUpper adj1
    vscale <- vScaleNew adj1
    scaleSetDefaultValues vscale
    boxPackStartDefaults topHBox vscale
    widgetShow vscale

    -- box for hscale and hscollbar
    --
    vbox <- vBoxNew False 10
    containerBorderWidth vbox 10
    boxPackStartDefaults topHBox vbox
    widgetShow vbox

    -- reuse the same adjustment
    --
    hscale <- hScaleNew adj1
    widgetSetUSize hscale 200 30
    scaleSetDefaultValues hscale
    boxPackStartDefaults vbox hscale
    widgetShow hscale

    -- reuse the same adjustment again
    --
    scrollbar <- hScrollbarNew adj1
    --
    -- notice how this causes the scales to always be updated
    -- continuously when the scrollbar is moved
    --
    rangeSetUpdatePolicy scrollbar UpdateContinuous
    boxPackStartDefaults vbox scrollbar
    widgetShow scrollbar

    -- box for the checkbutton
    --
    modBox <- hBoxNew False 10
    containerBorderWidth modBox 10
    boxPackStartDefaults bigBox modBox
    widgetShow modBox

    -- a checkbutton to control whether the value is displayed or not
    --
    button <- checkButtonNewWithLabel "Display value on scale widgets"
    toggleButtonSetState button True
    let cbDrawValue     :: CheckButton -> IO ()
	cbDrawValue btn  = do
			     active <- toggleButtonQueryState btn
			     scaleSetDrawValue hscale active
			     scaleSetDrawValue vscale active
        checkToggledHdl = ToggleButtonToggledHandler cbDrawValue
    signalConnect button checkToggledHdl
    boxPackStartDefaults modBox button
    widgetShow button

    -- RESTRICTION: we should have menu here, which determines the position
    --
    scaleSetValuePos hscale PosLeft
    scaleSetValuePos vscale PosLeft

    -- RESTRICTION: we should have another menu here, which determines the
    --		    policy 
    --
    rangeSetUpdatePolicy hscale UpdateDiscontinuous
    rangeSetUpdatePolicy vscale UpdateDiscontinuous

    -- box for the digit scale
    --
    dsBox <- hBoxNew False 10
    containerBorderWidth dsBox 10
    boxPackStartDefaults bigBox dsBox
    widgetShow dsBox

    -- a HScale widget for adjusting the number of digits on the
    -- sample scales
    --
    label <- labelNew "Scale Digits:"
    boxPackStartDefaults dsBox label
    widgetShow label

    adj2 <- adjustmentNew 1.0 0.0 5.0 1.0 1.0 0.0
    let cbDigitsScale     :: Adjustment -> IO ()
	cbDigitsScale adj  = do
			       val <- adjustmentGetValue adj
			       scaleSetDigits hscale (round val)
			       scaleSetDigits vscale (round val)
        cbDigitsScaleHdl = AdjustmentValueChangedHandler cbDigitsScale
    signalConnect adj2 cbDigitsScaleHdl
    scale <- hScaleNew adj2
    scaleSetDigits scale 0
    boxPackStartDefaults dsBox scale
    widgetShow scale

    -- box for the page size scale
    --
    psBox <- hBoxNew False 10
    containerBorderWidth psBox 10
    boxPackStartDefaults bigBox psBox
    widgetShow psBox

    --  and, one last HScale widget for adjusting the page size of the
    --  scrollbar
    --
    label <- labelNew "Scrollbar Page Size:"
    boxPackStartDefaults psBox label
    widgetShow label
    
    adj2 <- adjustmentNew 1.0 1.0 101.0 1.0 1.0 0.0
    let cbPageSize     :: Adjustment -> IO ()
	cbPageSize adj  = do
			    val <- adjustmentGetValue adj
			    --
			    -- set the page size and page increment size of
			    -- the sample adjustment to the value specified by
			    -- the "Page Size" scale
			    --
			    adjustmentSetPageSize      adj1 val
			    adjustmentSetPageIncrement adj1 val
			    --
			    -- now emit the "changed" signal to reconfigure
			    -- all the widgets that are attached to this
			    -- adjustment
			    --
			    adjustmentChanged adj1
			    return ()
        cbPageSizeHdl = AdjustmentValueChangedHandler cbPageSize
    signalConnect adj2 cbPageSizeHdl
    scale <- hScaleNew adj2
    scaleSetDigits scale 0
    boxPackStartDefaults psBox scale
    widgetShow scale

    separator <- hSeparatorNew
    boxPackStartDefaults bigBox separator
    widgetShow separator
    
    qvbox <- vBoxNew False 10
    containerBorderWidth qvbox 10
    boxPackStartDefaults bigBox qvbox
    widgetShow qvbox

    buttonQuit <- buttonNewWithLabel " Quit "
    let quitHandler = widgetDestroy window >> mainQuit
    signalConnect
      buttonQuit $ ButtonClickedHandler $ const quitHandler
    boxPackStartDefaults qvbox buttonQuit
    widgetSetFlags buttonQuit [CanDefault]
    widgetGrabDefault buttonQuit
    widgetShow buttonQuit

    widgetShow window

main :: IO ()
main  = do
	  Gtk.init Nothing
	  createRangeControl
	  Gtk.main
	  exitWith ExitSuccess
