-- tests GLib/GDK thread support with Haskell threads
--
-- As of GHC 4.02, the runtime system doesn't use OS threads for realizing
-- non-blocking I/O.  Thus, the `labelCounter' thread doesn't get any
-- opportunity to run while we are in `gtkMain'.
--

import System     (getArgs, ExitCode(ExitSuccess, ExitFailure), exitWith)

import Concurrent (forkIO)

import GLib       (threadInit)
import Gdk        (threadsEnter, threadsLeave)
import Gtk        hiding (main, init)
import qualified
       Gtk	  (main, init)


labelCounter         :: Label -> Int -> IO ()
labelCounter label n  = do
			  threadsEnter		-- async use of GTK+
			  labelSet label ("Label #" ++ show n)
			  threadsLeave		-- finished using GTK+
			  labelCounter label (n + 1)


main :: IO ()
main  = do
	  Gtk.init Nothing

          threadInit		-- this has to precede any `gdkThreadsXX' call

	  window <- windowNew WindowToplevel
	  windowSetTitle window "GTK+ Thread Test"
	  widgetSetUSize window 180 90
	  let winDelHdl = WidgetDeleteEventHandler $ \_ _ -> do
							       mainQuit
							       return False
	  signalConnect window winDelHdl
	  containerBorderWidth window 10

	  label <- labelNew "Initial label"
	  containerAdd window label
	  widgetShow label

	  widgetShow window

	  -- from here on, we are multi-threaded
	  --
	  forkIO (labelCounter label 0)

	  Gtk.main

	  exitWith ExitSuccess
